import ast
from ast import Add, BinOp, Constant, Expression, Mult, Name, Sub, expr, operator
from collections.abc import Mapping


def eval_binop(left_value: int, op: operator, right_value: int) -> int:
    match op:
        case Add():
            return left_value + right_value
        case Sub():
            return left_value - right_value
        case Mult():
            return left_value * right_value
        case _:
            raise ValueError("Unsupported operator")


def eval_expr(e: expr, m: Mapping[str, int]) -> int:
    match e:
        case Name(id):
            return m[id]
        case Constant(value) if type(value) is int:
            return value
        case BinOp(left, op, right):
            left_value: int = eval_expr(left, m)
            right_value: int = eval_expr(right, m)
            return eval_binop(left_value, op, right_value)
        case _:
            raise ValueError("Unsupported expression")


def eval(e: Expression, m: Mapping[str, int]) -> int:
    return eval_expr(e.body, m)


def main() -> None:
    e: Expression = ast.parse("(3 + (6 * 5)) * (2 + 7)", mode='eval')
    print(ast.dump(e, indent=4))
    print(eval(e, {}))


main()
